@@ -23,9 +23,9 @@
 #include "basic.h"
 
 
-#define NULLBITBOARD    ((uint64_t) 0)
-#define LIGHTBITBOARD	(Bitboard((uint64_t) 0x55aa55aa55aa55aa))
-#define DARKBITBOARD	(Bitboard((uint64_t) 0xaa55aa55aa55aa55))
+#define NULLBITBOARD	(Bitboard(0ULL))
+#define LIGHTBITBOARD	(Bitboard(0x55aa55aa55aa55aaULL))
+#define DARKBITBOARD	(Bitboard(0xaa55aa55aa55aa55ULL))
 
 
 class Bitboard
