@@ -1,8 +1,4 @@
-/* $Id: board_util.cc 1462 2007-12-18 20:49:56Z holger $
- *
- * HoiChess/board_util.cc
- *
- * Copyright (C) 2004, 2005 Holger Ruckdeschel <holger@hoicher.de>
+/* Copyright (C) 2004, 2005 Holger Ruckdeschel <holger@hoicher.de>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -58,35 +54,49 @@ bool Board::is_valid() const
 		return false;
 	
 	/* The opponent's king must not be in check. */
-	if (is_attacked(get_king(opponent), side))
+	if (is_attacked(get_king(XSIDE(side)), side))
 		return false;
 	
 	return true;
 }
 
+/*
+ * Detect draw by insufficient material.
+ * Only draw by rule is reported here. Positions (e.g. KNKN) that do have a
+ * mate that is very unlikely or cannot be forced are not reported as draw.
+ */
 bool Board::is_material_draw() const
 {
+	/* shortcuts */
+	Bitboard wn = get_knights(WHITE);
+	Bitboard bn = get_knights(BLACK);
+	Bitboard wb = get_bishops(WHITE);
+	Bitboard bb = get_bishops(BLACK);
+	Bitboard wbl = wb & LIGHTBITBOARD;
+	Bitboard wbd = wb & DARKBITBOARD;
+	Bitboard bbl = bb & LIGHTBITBOARD;
+	Bitboard bbd = bb & DARKBITBOARD;
+
 	if (material[WHITE] == 0 && material[BLACK] == 0) {
-		/* only kings left -> draw */
+		/* both sides have only their king -> draw */
 		return true;
-	} else if (get_pawns(WHITE) || get_pawns(BLACK)) {
-		/* there are still pawns -> no draw */
+	} else if (get_pawns(WHITE) || get_pawns(BLACK)
+				|| get_rooks(WHITE) || get_rooks(BLACK)
+				|| get_queens(WHITE) || get_queens(BLACK)) {
+		/* at least one side has a pawn, rook or queen -> no draw */
  		return false;
-	} else if ((material[WHITE] < mat_values[ROOK]
-			|| (material[WHITE] == 2 * mat_values[KNIGHT]
-				&& get_knights(WHITE).popcnt() == 2))
-		&& (material[BLACK] < mat_values[ROOK]
-			|| (material[BLACK] == 2 * mat_values[KNIGHT]
-				&& get_knights(BLACK).popcnt() == 2))) {
-		/* both sides have only a knight or a bishop -> draw
-		 * both sides have <= 2 knights only -> draw */
+	} else if ((wn | bn).popcnt() == 1 && !wb && !bb) {
+		/* KNK -> draw */
 		return true;
+	} else if ((wb | bb).popcnt() == 1 && !wn && !bn) {
+		/* KBK -> draw */
+		return true;
+	} else if (!wn && !bn && (!(wbl | bbl) || !(wbd | bbd))) {
+		/* KBKB with all bishops of same color -> draw */
+		return true;
+	} else {
+		return false;
 	}
-		
-	/* TODO  more */
-	
-	
-	return false;
 }
 
 void Board::print(FILE * fp, Move last_move) const
@@ -108,7 +118,7 @@ void Board::print(FILE * fp, Move last_move) const
 			
 			if (ansicolor && fp == stdout) {
 				const char * a1 = (i+j)%2
-					? "\033[47m" : "\033[40m";
+					? "\033[47m" : "\033[46m";
 
 				const char * a2 = "";
 				const char * a3 = "";
@@ -357,15 +367,19 @@ bool Board::parse_fen(const std::string & str)
 	return parse_fen(str.c_str());
 }
 
-Move Board::parse_move(const std::string & str) const
+Move Board::parse_move(const std::string & str, bool pseudolegal) const
 {
 	/*
 	 * Compare the input against all possible moves in coordinate
 	 * notation and SAN.
 	 */
+
 	Movelist moves;
 	generate_moves(&moves);
-	moves.filter_illegal(*this);
+	if (! pseudolegal) {
+		moves.filter_illegal(*this);
+	}
+
 	for (unsigned int i=0; i<moves.size(); i++) {
 		Move mov = moves[i];
 
