@@ -77,6 +77,7 @@ class Move {
 	inline bool is_promotion() const;
 	inline bool is_castle() const;
 	inline bool is_null() const;
+	inline bool is_irreversible() const;
 	inline int mat_gain() const;
 
       public:
@@ -204,6 +205,11 @@ inline bool Move::is_null() const
 	return (flags() & MOVE_NULL);
 }
 
+inline bool Move::is_irreversible() const
+{
+	return (ptype() == PAWN || is_capture() || is_castle());
+}
+
 inline int Move::mat_gain() const
 {
 	int gain = 0;
