@@ -22,16 +22,18 @@
 #include "version.h"
 #include "config.h"
 
-#if defined(HOICHESS)
-# ifdef VHDLCHESS
-#  define PROGNAME	"VHoiDLChess"
+#ifndef PROGNAME
+# if defined(HOICHESS)
+#  ifdef VHDLCHESS
+#   define PROGNAME	"VHoiDLChess"
+#  else
+#   define PROGNAME	"HoiChess"
+#  endif
+# elif defined(HOIXIANGQI)
+#  define PROGNAME	"HoiXiangqi"
 # else
-#  define PROGNAME	"HoiChess"
+#  error "neither HOICHESS nor HOIXIANGQI is defined"
 # endif
-#elif defined(HOIXIANGQI)
-# define PROGNAME	"HoiXiangqi"
-#else
-# error "neither HOICHESS nor HOIXIANGQI is defined"
 #endif
 
 #define AUTHOR		"Holger Ruckdeschel"
@@ -69,26 +71,28 @@
  */
 
 #ifdef WIN32
-
-#include <windows.h>
-
-#include <io.h>
-#define isatty _isatty
-
-#ifndef __MINGW32__
-# include "snprintf.h"
-#endif
-
-#include "strtok_r.h"
-
+# include <windows.h>
+# include <io.h>
+# define isatty _isatty
 #endif /* WIN32 */
 
 #ifdef __leonbare__
-# include "snprintf.h"
-# include "usleep.h"
+# include "sparc32/leon/printf.h"
+# include "sparc32/leon/usleep.h"
 #endif
 
-#include <unistd.h>
+
+/*
+ * Include own versions of library functions is required
+ */
+
+#ifndef HAVE_SNPRINTF
+# include "lib/snprintf.h"
+#endif
+
+#ifndef HAVE_STRTOK_R
+# include "lib/strtok_r.h"
+#endif
 
 
 /*
