@@ -20,7 +20,7 @@
 #include "common.h"
 #include "search.h"
 #include "shell.h"
-#ifdef USE_THREAD
+#ifdef WITH_THREAD
 # include "parallelsearch.h"
 #endif
 
@@ -29,7 +29,7 @@
 #include <unistd.h>
 
 
-#define SHOPT(x) (shell->get_options()->x)
+#define SHOPT(x) (shell->get_option_##x())
 
 /*****************************************************************************
  *
@@ -53,17 +53,12 @@ Search::Search(Shell * shell)
 	
 	maxdepth = MAXDEPTH;
 
-#ifdef USE_THREAD
+#ifdef WITH_THREAD
 	thread = NULL;
 #endif
 
 	stop = false;
 	timecheck_interval_nodes = 100;
-
-	ostat_nodes = 0;
-	ostat_csecs = 0;
-	ostat_depth_sum = 0;
-	ostat_depth_cnt = 0;
 }
 
 Search::~Search()
@@ -87,7 +82,7 @@ Search::~Search()
 Move Search::start(const Game * _game, Clock * _clock,
 		int _mode, Color _myside, unsigned int _maxdepth)
 {
-#ifdef USE_THREAD
+#ifdef WITH_THREAD
 	DBG(2, "locking start_mutex");
 	start_mutex.lock();
 	DBG(2, "locked start_mutex");
@@ -103,12 +98,11 @@ Move Search::start(const Game * _game, Clock * _clock,
 
 	stop = false;
 	Move best = main();
-	stop = false;
 
 	game = NULL;
 	clock = NULL;
 	
-#ifdef USE_THREAD
+#ifdef WITH_THREAD
 	DBG(2, "unlocking start_mutex");
 	start_mutex.unlock();
 	DBG(2, "unlocked start_mutex");
@@ -125,7 +119,7 @@ Move Search::start(const Board & board, const Clock & clock, int mode,
 	return start(&game, game.get_clock(), mode, NO_COLOR, maxdepth);
 }
 
-#ifdef USE_THREAD
+#ifdef WITH_THREAD
 void Search::start_thread(const Game * _game, Clock * _clock,
 		int _mode, Color _myside, unsigned int _maxdepth)
 {
@@ -260,50 +254,6 @@ int Search::stop_slave_thread()
 	return score;
 }
 
-/* Start a slave mode search without opening a thread, i.e. in the
- * thread context of the master search. This can be used to reduce
- * overhead when only a single slave is active, like at the bottom
- * of the search tree.
- */
-int Search::start_slave(const Game * game, Clock * clock, int mode,
-		Color myside, Node * node, unsigned int ply,
-		int depth, int extend,
-		int alpha, int beta)
-{
-	DBG(2, "locking start_mutex");
-	start_mutex.lock();
-	DBG(2, "locked start_mutex");
-
-	struct slave_search_args search_args;
-	search_args.node = node;
-	search_args.ply = ply;
-	search_args.depth = depth;
-	search_args.extend = extend;
-	search_args.alpha = alpha;
-	search_args.beta = beta;
-
-	ASSERT(this->game == NULL);
-	this->game = game;
-	this->clock = clock;
-	this->mode = mode;
-	this->myside = myside;
-
-	stop = false;
-	int score = slave_main(search_args);
-	stop = false;
-
-	this->game = NULL;
-	/* must not set this->clock = NULL, because clock is required
-	 * by print_statistics() which is called by master when slave
-	 * search is not running anymore */
-
-	DBG(2, "unlocking start_mutex");
-	start_mutex.unlock();
-	DBG(2, "unlocked start_mutex");
-
-	return score;
-}
-
 void * Search::thread_main(void * arg)
 {
 	struct thread_args * args = (struct thread_args *) arg;
@@ -332,16 +282,19 @@ void * Search::slave_thread_main(void * arg)
 	self->clock = args->clock;
 	self->mode = args->mode;
 	self->myside = args->myside;
+
 	args->score = self->slave_main(args->search_args);
+
 	self->game = NULL;
 	/* must not set self->clock = NULL, because clock is required
 	 * by print_statistics() which is called by master when slave
 	 * search is not running anymore */
+
 	args->master->slave_ready(self);	
 
 	return args;
 }
-#endif // USE_THREAD
+#endif // WITH_THREAD
 
 void Search::interrupt()
 {
@@ -359,12 +312,15 @@ void Search::interrupt()
 
 Move Search::main()
 {
-#ifdef USE_THREAD
+#ifdef WITH_THREAD
 	DBG(2, "locking main_mutex");
 	main_mutex.lock();
 	DBG(2, "locked main_mutex");
 #endif
 
+	slave = false;
+
+	last_timecheck_csecs = 0;
 	next_timecheck_nodes = timecheck_interval_nodes;
 	next_update_csecs = 0;
 
@@ -373,10 +329,22 @@ Move Search::main()
 	histtable[WHITE]->reset();
 	histtable[BLACK]->reset();
 
+	/* Convert the game entries into a list of node. The
+	 * last one will be the root node of the search tree. */
+	const std::list<GameEntry>& gameentries = game->get_entries();
+	NodeAllocator gamenodealloc(gameentries.size() + 1);
+	Node openingnode(game->get_opening());
+	Node * node = &openingnode;
+	for (std::list<GameEntry>::const_iterator it = gameentries.begin();
+			it != gameentries.end(); it++) {
+		node = node->make_move(it->get_move(), &gamenodealloc);
+	}
+	rootnode = node;
+
 	/* Initialize the root node. init_root() already assigns one legal
 	 * move as best, in case search terminates without choosing a move. */
-	rootnode = new Node();
-	rootnode->init_root(game->get_board());
+	ASSERT(rootnode->get_board() == game->get_board());
+	rootnode->init_root();
 	rootdepth = 0;
 
 	/* If we're not supplied with a clock, create a local one just for
@@ -404,14 +372,11 @@ Move Search::main()
 
 	if (verbose) {
 		print_statistics();
-		update_overall_statistics();
-		print_overall_statistics();
 	}
 	
-	delete rootnode;
 	rootnode = NULL;
 	
-#ifdef USE_THREAD
+#ifdef WITH_THREAD
 	DBG(2, "unlocking main_mutex");
 	main_mutex.unlock();
 	DBG(2, "unlocked main_mutex");
@@ -427,14 +392,17 @@ Move Search::main()
  *
  *****************************************************************************/
 
-#ifdef USE_THREAD
+#ifdef WITH_THREAD
 int Search::slave_main(const struct slave_search_args& search_args)
 {
 	DBG(2, "locking main_mutex");
 	main_mutex.lock();
 	DBG(2, "locked main_mutex");
 
+	slave = true;
+
 	/* clock is passed from master and set by slave_thread_main */
+	last_timecheck_csecs = 0;
 	next_timecheck_nodes = timecheck_interval_nodes;
 	next_update_csecs = 0;
 
@@ -462,7 +430,7 @@ int Search::slave_main(const struct slave_search_args& search_args)
 
 	return score;
 }
-#endif /* USE_THREAD */
+#endif /* WITH_THREAD */
 
 /*****************************************************************************
  *
@@ -473,6 +441,9 @@ int Search::slave_main(const struct slave_search_args& search_args)
 /* Aspiration window for iterative deepening */
 #define WINDOW	50
 
+/* Shortcut */
+#define failsoft (shell->get_option_search_failsoft())
+
 Move Search::iterate(unsigned int depth)
 {
 	Move best = rootnode->get_best_move(); /* set by Node::init_root() */
@@ -493,7 +464,7 @@ again:
 		iteration_start_csecs = Clock::to_cs(clock->get_elapsed_time());
 
 		score = search_root(rootnode, 0, rootdepth, alpha, beta);
-		if (score < alpha || score > beta) {
+		if (!failsoft && (score < alpha || score > beta) && !stop) {
 			printf("iterate(): fail hard condition violated:"
 					" score=%d alpha=%d beta=%d\n",
 					score, alpha, beta);
@@ -526,7 +497,7 @@ again:
 				break;
 			}
 			alpha = -INFTY;
-			beta = beta;
+			//beta = beta;
 			goto again;
 		} else if (score >= beta && score < MATE) {
 			print_result(rootdepth, score, searchresult::FAILHIGH,
@@ -534,7 +505,7 @@ again:
 			if (stop_iteration) {
 				break;
 			}
-			alpha = alpha;
+			//alpha = alpha;
 			beta = INFTY;
 			goto again;
 		} else {
@@ -575,9 +546,10 @@ int Search::search_root(Node * node, unsigned int ply, int depth,
 
 //	DBG(3, "depth=%d, alpha=%d, beta=%d\n", depth, alpha, beta);
 	
+	int save_alpha = alpha;
 	int score;
+	int bestscore = -INFTY;
 	int moves = 0;
-	bool first = true;
 	for (Move mov = node->first(); mov; mov = node->next()) {
 		/* make move */
 		Node * child = node->make_move(mov, &nodealloc);
@@ -597,11 +569,10 @@ int Search::search_root(Node * node, unsigned int ply, int depth,
 
 		/* Search the current move. We use a standard
 		 * principal variation search here. */
-		if (first) {
+		if (alpha <= save_alpha) { /* no PV move found yet */
 			score = -search(child, ply+1, depth-1, 0,
 					-beta, -alpha);
-			first = false;
-		} else {
+		} else { /* PV move has already been found */
 			score = -search(child, ply+1, depth-1, 0,
 					-alpha-1, -alpha);
 			if (score > alpha && score < beta) {
@@ -610,7 +581,7 @@ int Search::search_root(Node * node, unsigned int ply, int depth,
 			}			
 		}
 
-		if (score < alpha || score > beta) {
+		if (!failsoft && (score < alpha || score > beta) && !stop) {
 			printf("search_root(): fail hard condition violated:"
 					" score=%d alpha=%d beta=%d\n",
 					score, alpha, beta);
@@ -625,14 +596,14 @@ int Search::search_root(Node * node, unsigned int ply, int depth,
 			 * result of the previous (completed) iteration.
 			 * For that reason, we can return an arbitrary value
 			 * here, because iterate() won't use it anyway. */
-			return alpha;
+			return INT_MIN;
 		}
 		
 		/* this is taken for move ordering at the next iteration */
 		node->set_current_score(score);
 		
-		if (score > alpha) {
-			alpha = score;
+		if (score > bestscore) {
+			bestscore = score;
 			node->set_best(mov, child);
 			print_result(depth, score, searchresult::INTERMEDIATE,
 					node->get_best_line());
@@ -640,9 +611,14 @@ int Search::search_root(Node * node, unsigned int ply, int depth,
 
 		child->free();
 
+		if (score > alpha) {
+			alpha = score;
+		}
 
 		if (score >= beta) {
-			score = beta; /* fail hard */
+			if (!failsoft) {
+				score = beta; /* fail hard */
+			}
 			STAT_INC(stat_cut);
 			break;
 		}
@@ -655,8 +631,11 @@ int Search::search_root(Node * node, unsigned int ply, int depth,
 	stat_moves_cnt++;
 #endif
 
-//	DBG(3, "return alpha=%d\n", alpha);
-	return alpha;
+	if (failsoft) {
+		return bestscore;
+	} else {
+		return alpha;
+	}
 }
 
 int Search::search(Node * node, unsigned int ply, int depth, int extend,
@@ -667,22 +646,19 @@ int Search::search(Node * node, unsigned int ply, int depth, int extend,
 		return quiescence_search(node, ply, alpha, beta);
 	}
 
-	if (is_draw(node)) {
-		int drawscore = DRAW;
-		/* strict fail hard */
-		if (drawscore < alpha) {
-			return alpha;
-		} else if (drawscore > beta) {
-			return beta;
-		} else {
+	int drawscore;
+	if (is_draw(node, ply, &drawscore)) {
+		if (failsoft) {
 			return drawscore;
+		} else { /* strict fail hard */
+			return bound_score(drawscore, alpha, beta);
 		}
 	}
 
 	int save_alpha = alpha; /* need old value to save in hash table */
 	int score;
+	int bestscore = -INFTY;
 	int moves = 0;
-	bool first = true;
 	nodes_fullwidth++;
 	
 	if (ply > maxplyreached_fullwidth) {
@@ -703,7 +679,11 @@ int Search::search(Node * node, unsigned int ply, int depth, int extend,
 		child->free();
 		if (score >= beta) {
 			STAT_INC(stat_nullcut);
-			return beta;
+			if (failsoft) {
+				return score;
+			} else {
+				return beta; /* fail hard */
+			}
 		}
 	}
 
@@ -773,11 +753,10 @@ int Search::search(Node * node, unsigned int ply, int depth, int extend,
 
 		/* Search the current move. We use a standard
 		 * principal variation search here. */
-		if (first) {
+		if (alpha <= save_alpha) { /* no PV move found yet */
 			score = -search(child, ply+1, depth-1, extend,
 					-beta, -alpha);
-			first = false;
-		} else {
+		} else { /* PV move has already been found */
 			score = -search(child, ply+1, depth-1, extend,
 					-alpha-1, -alpha);
 			if (score > alpha && score < beta) {
@@ -786,27 +765,38 @@ int Search::search(Node * node, unsigned int ply, int depth, int extend,
 			}			
 		}
 
-		if (score < alpha || score > beta) {
+		if (!failsoft && (score < alpha || score > beta) && !stop) {
 			printf("search(): fail hard condition violated:"
 					" score=%d alpha=%d beta=%d\n",
 					score, alpha, beta);
 		}
 
-		check_time();
+		check_time(false, false);
 		if (stop) {
 			child->free();
-			return alpha;
+			/* We don't get any useful result after canceling the
+			 * alpha-beta search, so iterate() must return the
+			 * result of the previous (completed) iteration.
+			 * For that reason, we can return an arbitrary value
+			 * here, because iterate() won't use it anyway. */
+			return INT_MIN;
 		}
 
-		if (score > alpha) {
-			alpha = score;
+		if (score > bestscore) {
+			bestscore = score;
 			node->set_best(mov, child);
 		}
 
 		child->free();
 
+		if (score > alpha) {
+			alpha = score;
+		}
+
 		if (score >= beta) {
-			score = beta; /* fail hard */
+			if (!failsoft) {
+				score = beta; /* fail hard */
+			}
 			STAT_INC(stat_cut);
 			break;
 		}
@@ -821,18 +811,19 @@ int Search::search(Node * node, unsigned int ply, int depth, int extend,
 #else
 # error "neither HOICHESS nor HOIXIANGQI is defined"
 #endif
-		/* strict fail hard */
-		if (matescore < alpha) {
-			alpha = alpha;
-		} else if (matescore > beta) {
-			alpha = beta;
-		} else {
-			alpha = matescore;
+		if (failsoft) {
+			bestscore = matescore;
+		} else { /* strict fail hard */
+			alpha = bound_score(matescore, alpha, beta);
 		}
 	}
 
 	/* Save search result in hash table. */
-	store_hashtable(node, depth, save_alpha, beta, alpha);
+	if (failsoft) {
+		store_hashtable(node, depth, save_alpha, beta, bestscore);
+	} else {
+		store_hashtable(node, depth, save_alpha, beta, alpha);
+	} 
 	
 	add_history(node);
 	add_killer(node);
@@ -842,7 +833,11 @@ int Search::search(Node * node, unsigned int ply, int depth, int extend,
 	stat_moves_cnt++;
 #endif
 	
-	return alpha;
+	if (failsoft) {
+		return bestscore;
+	} else {
+		return alpha;
+	}
 }
 
 int Search::quiescence_search(Node * node, unsigned int ply, 
@@ -850,6 +845,7 @@ int Search::quiescence_search(Node * node, unsigned int ply,
 {
 	int save_alpha = alpha; /* need old value to save in hash table */
 	int score = 0; /* initialized only to avoid compiler warning */
+	int bestscore = -INFTY;
 	int moves = 0;
 	
 	nodes_quiesce++;
@@ -870,10 +866,18 @@ int Search::quiescence_search(Node * node, unsigned int ply,
 	 */
 	score = evaluator->eval(node->get_board(), alpha, beta, myside);
 	if (score >= beta) {
-		return beta; /* fail hard */
-	} else if (score > alpha) {
+		if (failsoft) {
+			return score;
+		} else {
+			return beta; /* fail hard */
+		}
+	}
+	if (score > alpha) {
 		alpha = score;
 	}
+	if (failsoft) {
+		bestscore = score;
+	}
 
 	/* MAXPLY is an absolute depth limit */
 	if (ply == MAXPLY-1) {
@@ -893,22 +897,38 @@ int Search::quiescence_search(Node * node, unsigned int ply,
 		
 		score = -quiescence_search(child, ply+1, -beta, -alpha);
 
-		if (score < alpha || score > beta) {
+		if (!failsoft && (score < alpha || score > beta) && !stop) {
 			printf("q_search(): fail hard condition violated:"
 					" score=%d alpha=%d beta=%d\n",
 					score, alpha, beta);
 		}
 
+		check_time(false, false);
+		if (stop) {
+			child->free();
+			/* We don't get any useful result after canceling the
+			 * alpha-beta search, so iterate() must return the
+			 * result of the previous (completed) iteration.
+			 * For that reason, we can return an arbitrary value
+			 * here, because iterate() won't use it anyway. */
+			return INT_MIN;
+		}
 
-		if (score > alpha) {
-			alpha = score;
+		if (score > bestscore) {
+			bestscore = score;
 			node->set_best(mov, child);
 		}
 
 		child->free();
 
+		if (score > alpha) {
+			alpha = score;
+		}
+
 		if (score >= beta) {
-			score = beta; /* fail hard */
+			if (!failsoft) {
+				score = beta; /* fail hard */
+			}
 			STAT_INC(stat_cut);
 			break;
 		}
@@ -917,64 +937,133 @@ int Search::quiescence_search(Node * node, unsigned int ply,
 	/* Test for checkmate */
 	if (moves == 0 && node->in_check()) {
 		int matescore = -INFTY + ply;
-		/* strict fail hard */
-		if (matescore < alpha) {
-			alpha = alpha;
-		} else if (matescore > beta) {
-			alpha = beta;
-		} else {
-			alpha = matescore;
+		if (failsoft) {
+			bestscore = matescore;
+		} else { /* strict fail hard */
+			alpha = bound_score(matescore, alpha, beta);
 		}
 	}
 	
 	/* Save search result in hash table. */
-	store_hashtable(node, 0, save_alpha, beta, alpha);
+	if (failsoft) {
+		store_hashtable(node, 0, save_alpha, beta, bestscore);
+	} else {
+		store_hashtable(node, 0, save_alpha, beta, alpha);
+	}
 
 #ifdef COLLECT_STATISTICS
 	stat_moves_sum_quiesce += moves;
 	stat_moves_cnt_quiesce++;
 #endif
 	
-	return alpha;
+	if (failsoft) {
+		return bestscore;
+	} else {
+		return alpha;
+	}
 }
 
 /* 
- * Return true if the current position is a draw (draw by rule).
+ * Return true if the current position is a draw by rule. The score assigned
+ * to the draw is returned by the score output parameter.
+ * This is because the code also handles perpetual check for xiangqi, in which
+ * case a mate score is assigned. It could also be used for chess to assign
+ * different draw scores in different situations.
  */
-bool Search::is_draw(const Node * node)
+bool Search::is_draw(const Node * node, unsigned int ply, int * score)
 {
 	const Board & board = node->get_board();
 	
 	/* 50 move rule */
 	if (board.get_movecnt50() >= 100) {
+		*score = DRAW;
 		return true;
 	}
 	
 	/* Draw due to insufficient material */
 	if (board.is_material_draw()) {
+		*score = DRAW;
 		return true;
 	}
 	
-	/* Look for repetitions in the tree */
-	int rep = 1;
-	for (const Node * p = node->get_parent();
-				p != NULL; p = p->get_parent()) {
-		if (p->get_board().get_hashkey() == board.get_hashkey()) {
+	/* Look for repetitions */
+	if (is_repetition(node, ply, score)) {
+		return true;
+	}
+
+	return false;
+}
+
+/* 
+ * Return true if the current position is a repetition. The score assigned
+ * to the repetition is returned by the score output parameter.
+ *
+ * For xiangqi, this also handled perpetual check: If between the repeating
+ * positions, one side always checks but the other side does not, a mate
+ * score is returned.
+ */
+bool Search::is_repetition(const Node * node, unsigned int ply, int * score)
+{
+#ifndef HOIXIANGQI
+	(void) ply;
+#endif
+
+	/* Count repetitions until next irreversible move. */
+	int rep = 0;
+
+#ifdef HOIXIANGQI
+	unsigned int moves[2] = {0, 0}; /* [0] is side */
+	unsigned int checks[2] = {0, 0}; /* [0] is side */
+	/* in the current position, xside has moved last */
+	unsigned int side = 1;
+#endif
+
+	for (const Node * p = node; p != NULL; p = p->get_parent()) {
+		if (p->get_played_move().is_irreversible()) {
+			break;
+		}
+
+		if (p != node && p->get_board().get_hashkey() 
+				== node->get_board().get_hashkey()) {
 			rep++;
 		}
-		
-		if (rep >= 2) {
-			return true;
+
+#ifdef HOIXIANGQI
+		moves[side]++;
+		if (p->in_check()) {
+			checks[side]++;
+		}
+		side = 1 - side;
+#endif
+
+		if (rep) {
+			break;
 		}
 	}
 
-	/* Look for repetitions in the game history */
-	rep += game->repetitions_search(board);
-	if (rep >= 2) {
-		return true; 
+	if (rep) {
+#ifdef HOIXIANGQI
+		if (moves[0] > 0 && checks[0] == moves[0]
+				&& moves[1] > 0 && checks[1] == moves[1]) {
+			*score = DRAW;
+			printf("perpetual both, %u/%u\n", checks[0], checks[1]);
+		} else if (moves[0] > 0 && checks[0] == moves[0]) {
+			*score = -INFTY + ply;
+			printf("perpetual side, %u/%u\n", checks[0], checks[1]);
+		} else if (moves[1] > 0 && checks[1] == moves[1]) {
+			*score = INFTY - ply;
+			printf("perpetual xside, %u/%u\n", checks[0],checks[1]);
+		} else {
+			*score = DRAW;
+		}
+#else
+		*score = DRAW;
+#endif
+		return true;
+	} else {
+		*score = INT_MIN; /* never used */
+		return false;
 	}
-	
-	return false;
 }
 
 /* Returns true if the probe result can be directly used as search
@@ -986,7 +1075,13 @@ bool Search::probe_hashtable(Node * node, int depth, int alpha, int beta,
 		return false;
 	}
 
-	struct Node::pvline pvline;
+	/* Constructor of Node::pvline is expensive as it contains a
+	 * quite large array of Move elements. So we add a pvline as
+	 * class member to avoid construction each time probe_hashtable()
+	 * runs. */
+	//struct Node::pvline pvline;
+	struct Node::pvline & pvline = _probe_hashtable_pvline;
+
 	const HashEntry * entry = hashtable->probe(node->get_board(), &pvline);
 	if (!entry) {
 		return false;
@@ -1005,23 +1100,19 @@ bool Search::probe_hashtable(Node * node, int depth, int alpha, int beta,
 		switch (entry->get_type()) {
 		case HashEntry::EXACT:
 			hashtable->incr_hits2();
-			/* strict fail hard */
-			if (s < alpha) {
-				*score = alpha;
-			} else if (s > beta) {
-				*score = beta;
-			} else {
+			if (failsoft) {
 				*score = s;
+			} else { /* strict fail hard */
+				*score = bound_score(s, alpha, beta);
 			}
 			return true;
 		case HashEntry::ALPHA:
 			if (s <= alpha) {
 				hashtable->incr_hits2();
-				/* strict fail hard */
-				if (s < alpha) {
-					*score = alpha;
-				} else {
+				if (failsoft) {
 					*score = s;
+				} else { /* strict fail hard */
+					*score = bound_score(s, alpha, beta);
 				}
 				return true;
 			}
@@ -1029,11 +1120,10 @@ bool Search::probe_hashtable(Node * node, int depth, int alpha, int beta,
 		case HashEntry::BETA:
 			if (s >= beta) {
 				hashtable->incr_hits2();
-				/* strict fail hard */
-				if (s > beta) {
-					*score = beta;
-				} else {
+				if (failsoft) {
 					*score = s;
+				} else { /* strict fail hard */
+					*score = bound_score(s, alpha, beta);
 				}
 				return true;
 			}
@@ -1102,60 +1192,92 @@ void Search::add_killer(Node * node)
 	}
 }
 
+int Search::bound_score(int score, int alpha, int beta)
+{
+	if (score > beta) {
+		return beta;
+	} else if (score < alpha) {
+		return alpha;
+	} else {
+		return score;
+	}
+}
+
 void Search::check_time(bool force_check, bool force_update)
 {
+	/* check time only after a certain number of nodes to reduce
+	 * system call overhead */
 	unsigned long long nodes = nodes_fullwidth + nodes_quiesce;
 	if (nodes < next_timecheck_nodes && !force_check) {
 		return;
 	}
 
+	/* check time */
 	if (clock->timeout()) {
 		stop = true;
+		return;
 	}
 
-	/* NOTE: In slave mode, we do not print thinking output,
-	 * nor do we decide about time extensions.
-	 * There is currently no slave mode flag, so we check
-	 * for rootnode != NULL instead. (TODO) */
+	unsigned long elapsed_csecs = Clock::to_cs(clock->get_elapsed_time());
 
-	unsigned int elapsed_csecs = Clock::to_cs(clock->get_elapsed_time());
-	if (elapsed_csecs >= next_update_csecs || force_update) {
+	/* update and adjust time check interval */
+	if (!force_check) {
+		adjust_timecheck_interval(elapsed_csecs);
+	}
+	last_timecheck_csecs = elapsed_csecs;
+	next_timecheck_nodes = nodes + timecheck_interval_nodes;
+
+	/* thinking output (not in slave mode) */
+	if (!slave && (elapsed_csecs >= next_update_csecs || force_update)) {
 		next_update_csecs = elapsed_csecs
 			+ SHOPT(search_update_interval_csecs);
-		if (rootnode != NULL) {
-			print_thinking(rootdepth);
-		}
+		print_thinking(rootdepth);
 	}
 
-	if (mode == MOVE && rootnode != NULL
+	/* time extensions (not in slave mode) */
+	if (mode == MOVE && !slave && !clock->is_exact()
 			&& SHOPT(search_extend_time_iteration_enable)) {
 		extend_time_iteration();
 	}
+}
+	
+/* 
+ * Adjust timecheck_interval_nodes based on time since last check.
+ */
+void Search::adjust_timecheck_interval(unsigned long elapsed_csecs)
+{
+	unsigned long diff = elapsed_csecs - last_timecheck_csecs;
+	unsigned long dmin = SHOPT(search_timecheck_interval_csecs_min);
+	unsigned long dmax = SHOPT(search_timecheck_interval_csecs_max);
 
-	/* adjust timecheck_interval_nodes based on time since last check */
-	if (!force_check) {
-		unsigned int diff = elapsed_csecs - last_timecheck_csecs;
-		unsigned int dmin = SHOPT(search_timecheck_interval_csecs_min);
-		unsigned int dmax = SHOPT(search_timecheck_interval_csecs_max);
-
-		if (dmax == 0) {
-			timecheck_interval_nodes = 1;
-		} else if (diff < dmin
-				&& timecheck_interval_nodes < ULONG_MAX/2) {
-			timecheck_interval_nodes *= 2;
-		} else if (diff > dmax && timecheck_interval_nodes > 1) {
-			timecheck_interval_nodes /= 2;
-		}
-		DBG(2, "elapsed_csecs=%u last_timecheck_csecs=%lu diff=%u"
+	if (dmax == 0) {
+		timecheck_interval_nodes = 1;
+		DBG(1, "elapsed_csecs=%u last_timecheck_csecs=%lu diff=%u"
+				" timecheck_interval_nodes=%lu\n",
+				elapsed_csecs, last_timecheck_csecs, diff,
+				timecheck_interval_nodes);
+	} else if (diff < dmin
+			&& timecheck_interval_nodes < ULONG_MAX/2) {
+		timecheck_interval_nodes *= 2;
+		DBG(1, "elapsed_csecs=%u last_timecheck_csecs=%lu diff=%u"
+				" timecheck_interval_nodes=%lu\n",
+				elapsed_csecs, last_timecheck_csecs, diff,
+				timecheck_interval_nodes);
+	} else if (diff > dmax && timecheck_interval_nodes > 1) {
+		timecheck_interval_nodes /= 2;
+		DBG(1, "elapsed_csecs=%u last_timecheck_csecs=%lu diff=%u"
 				" timecheck_interval_nodes=%lu\n",
 				elapsed_csecs, last_timecheck_csecs, diff,
 				timecheck_interval_nodes);
 	}
-
-	last_timecheck_csecs = elapsed_csecs;
-	next_timecheck_nodes = nodes + timecheck_interval_nodes;
+	
+	DBG(2, "elapsed_csecs=%u last_timecheck_csecs=%lu diff=%u"
+			" timecheck_interval_nodes=%lu\n",
+			elapsed_csecs, last_timecheck_csecs, diff,
+			timecheck_interval_nodes);
 }
 
+
 /*
  * Extend search time when remaining time is expected to be
  * slightly too short to finish current iteration.
