@@ -19,7 +19,7 @@
 
 #include "common.h"
 #include "shell.h"
-#ifdef USE_THREAD
+#ifdef WITH_THREAD
 # include "parallelsearch.h"
 #endif
 #include "epd.h"
@@ -63,7 +63,7 @@ const struct Shell::command Shell::shell_commands[] = {
 	{ "bk",		&Shell::cmd_bk,		""	},
 	{ "undo",	&Shell::cmd_undo,	""	},
 	{ "remove",	&Shell::cmd_remove,	""	},
-#ifdef USE_THREAD
+#ifdef WITH_THREAD
 	{ "hard",	&Shell::cmd_hard,	"Turn on pondering (thinking on opponent's time)" },
 	{ "easy",	&Shell::cmd_easy,	"Turn off pondering" },
 #else
@@ -72,7 +72,7 @@ const struct Shell::command Shell::shell_commands[] = {
 #endif
 	{ "post",	&Shell::cmd_post,	"Show thinking output" },
 	{ "nopost",	&Shell::cmd_nopost,	"Hide thinking output" },
-#ifdef USE_THREAD
+#ifdef WITH_THREAD
 	{ "analyze",	&Shell::cmd_analyze,	"Enter analysis mode" },
 	{ "exit",	&Shell::cmd_exit,	"Leave analysis mode" },
 #endif
@@ -83,7 +83,7 @@ const struct Shell::command Shell::shell_commands[] = {
 	{ "pause",	NULL,			""	},
 	{ "resume",	NULL,			""	},
 	{ ".",		&Shell::cmd_null,	""	},
-#ifdef USE_THREAD
+#ifdef WITH_THREAD
 	{ "cores",	&Shell::cmd_cores,	""	},
 #endif
 	{ "option",	&Shell::cmd_option,	""	},
@@ -108,6 +108,7 @@ const struct Shell::command Shell::shell_commands[] = {
 	{ "savegame",	&Shell::cmd_savegame,	""	},
 	{ "redo",	&Shell::cmd_redo,	""	},
 	{ "options",	&Shell::cmd_options,	"List all options and their current values"	},
+	{ "atexit",	&Shell::cmd_atexit,	""	},
 	
 	{ NULL, NULL, NULL }
 };
@@ -154,14 +155,14 @@ int Shell::cmd_protover()
 	
 	ss << " ping=1 setboard=1 time=1 sigint=0 sigterm=0 colors=0";
 
-#ifdef USE_THREAD
+#ifdef WITH_THREAD
 	ss << " analyze=1";
 #else
 	ss << " analyze=0";
 #endif
 
 	ss << " name=1";
-#ifdef USE_THREAD
+#ifdef WITH_THREAD
 	ss << " smp=1";
 #endif
 	
@@ -481,7 +482,7 @@ int Shell::cmd_nopost()
 	return SHELL_CMD_OK;
 }
 
-#ifdef USE_THREAD
+#ifdef WITH_THREAD
 int Shell::cmd_analyze()
 {
 	stop_search();
@@ -497,9 +498,9 @@ int Shell::cmd_exit()
 	flag_analyze = false;
 	return SHELL_CMD_OK;
 }
-#endif // USE_THREAD
+#endif // WITH_THREAD
 
-#ifdef USE_THREAD
+#ifdef WITH_THREAD
 /*
  * cores 1         activates standard search
  * cores N, N>=2   activates parallel search with N threads
@@ -550,7 +551,7 @@ int Shell::cmd_cores()
 
 	return SHELL_CMD_OK;
 }
-#endif // USE_THREAD
+#endif // WITH_THREAD
 
 int Shell::cmd_option()
 {
@@ -568,10 +569,12 @@ int Shell::cmd_option()
 	for (std::list<struct option>::iterator it = option_list.begin();
 			it != option_list.end(); it++) {
 		if (it->name == name) {
-			if (sscanf(val.c_str(), "%d", it->varptr) != 1) {
+			int tmp;
+			if (sscanf(val.c_str(), "%d", &tmp) != 1) {
 				printf("Illegal value: %s\n", val.c_str());
 				return SHELL_CMD_FAIL;
 			}
+			*it->varptr = tmp;
 			return SHELL_CMD_OK;
 		}
 	}
@@ -1210,3 +1213,14 @@ int Shell::cmd_options()
 	}
 	return SHELL_CMD_OK;
 }
+
+int Shell::cmd_atexit()
+{
+	SHELL_CMD_REQUIRE_ARGS(1);
+
+	std::vector<std::string> tmp = cmd_args;
+	tmp.erase(tmp.begin());
+	atexit_commands.push_back(tmp);
+
+	return SHELL_CMD_OK;
+}
