@@ -19,7 +19,7 @@
 
 #include "common.h"
 #include "debug.h"
-#ifdef USE_THREAD
+#ifdef WITH_THREAD
 # include "mutex.h"
 # include "thread.h"
 #endif
@@ -42,7 +42,7 @@ void __debug_helper::__dbg(unsigned int level, const char * fmt, ...)
 		return;
 	}
 
-#ifdef USE_THREAD
+#ifdef WITH_THREAD
 	if (mutex) {
 		mutex->lock();
 	}
@@ -58,7 +58,7 @@ void __debug_helper::__dbg(unsigned int level, const char * fmt, ...)
 		fprintf(fp, "\n");
 	}
 
-#ifdef USE_THREAD
+#ifdef WITH_THREAD
 	if (mutex) {
 		mutex->unlock();
 	}
@@ -67,7 +67,7 @@ void __debug_helper::__dbg(unsigned int level, const char * fmt, ...)
 
 void __debug_helper::__warn(const char * fmt, ...)
 {
-#ifdef USE_THREAD
+#ifdef WITH_THREAD
 	if (mutex) {
 		mutex->lock();
 	}
@@ -83,7 +83,7 @@ void __debug_helper::__warn(const char * fmt, ...)
 		fprintf(fp, "\n");
 	}
 
-#ifdef USE_THREAD
+#ifdef WITH_THREAD
 	if (mutex) {
 		mutex->unlock();
 	}
@@ -92,7 +92,7 @@ void __debug_helper::__warn(const char * fmt, ...)
 
 void __debug_helper::__bug(const char * fmt, ...)
 {
-#ifdef USE_THREAD
+#ifdef WITH_THREAD
 	if (mutex) {
 		mutex->lock();
 	}
@@ -111,7 +111,7 @@ void __debug_helper::__bug(const char * fmt, ...)
 	fprintf(fp, "This is a bug in %s. Please report it to %s.\n",
 			PROGNAME, AUTHOR_EMAIL);
 
-#ifdef USE_THREAD
+#ifdef WITH_THREAD
 	if (mutex) {
 		mutex->unlock();
 	}
