@@ -1,8 +1,4 @@
-/* $Id: init.cc 1462 2007-12-18 20:49:56Z holger $
- *
- * HoiChess/init.cc
- *
- * Copyright (C) 2004-2007 Holger Ruckdeschel <holger@hoicher.de>
+/* Copyright (C) 2004-2007 Holger Ruckdeschel <holger@hoicher.de>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,6 +23,15 @@
 
 #include <time.h>
 
+#ifdef __leonbare__
+extern "C" {
+# include <asm-leon/timer.h>
+# define SOLARIS_NP
+# include <fsu_pthread.h>
+}
+#endif
+
+
 
 /*****************************************************************************
  * 
@@ -37,7 +42,11 @@
 
 void init()
 {
-	basic_init();
+#ifdef __leonbare__
+	//leonbare_init_ticks();
+	pthread_init();
+#endif
+
 #ifdef HOICHESS
 	Bitboard::init();
 #endif
@@ -57,10 +66,23 @@ void init()
 #endif
 }
 
-void fini()
-{
-#ifdef STATS_MOVELIST
-	Movelist::print_stats();
-#endif
-}	
 
+/*****************************************************************************
+ * 
+ * Default builtin initial commands.
+ * Declared with weak linkage so that they can be overridden at build
+ * time.
+ *
+ * This is for embedded systems where command line arguments (e.g. --initcmd)
+ * or script files are not available.
+ *
+ *****************************************************************************/
+
+const char * builtin_initcmds[]
+#ifndef WIN32
+__attribute__((weak))
+#endif
+	= { NULL }; /* {
+	"echo default builtin initcmd",
+	NULL
+};*/
