@@ -24,17 +24,15 @@
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
-
 #include <string>
 
-
-#ifdef USE_THREAD
+#ifdef WITH_THREAD
 extern Mutex stdout_mutex;
 extern int atomic_printf(const char * fmt, ...);
 extern int atomic_fprintf(FILE * fp, Mutex * mutex, const char * fmt, ...);
-#else // !USE_THREAD
+#else // !WITH_THREAD
 #define atomic_printf printf
-#endif // !USE_THREAD
+#endif // !WITH_THREAD
 
 
 extern uint64_t random64();
@@ -42,6 +40,9 @@ extern std::string strprintf(const char * fmt, ...);
 
 extern bool parse_size(const char * s, ssize_t * n);
 
+extern unsigned long long get_realtime_us();
+extern unsigned long long get_cputime_us();
+
 
 #define MAX(a,b) ((a) > (b) ? (a) : (b))
 #define MIN(a,b) ((a) < (b) ? (a) : (b))
