@@ -62,6 +62,7 @@ class Move {
 	inline bool is_normal() const;
 	inline bool is_capture() const;
 	inline bool is_null() const;
+	inline bool is_irreversible() const;
 	inline int mat_gain() const;
 
       public:
@@ -156,6 +157,12 @@ inline bool Move::is_null() const
 	return (flags() & MOVE_NULL);
 }
 
+inline bool Move::is_irreversible() const
+{
+	return ((ptype() == PAWN && RNK(from()) != RNK(to())) // P forward move
+			|| is_capture());
+}
+
 inline int Move::mat_gain() const
 {
 	int gain = 0;
